unit MinModal;

{
MinModal: minimizer for modal windows. Why we can't minimize an
          application when a modal window is active? Drop this
          component to the form, open form by method 'ShowModal'
          and try minimize this form at run time.
          All your application will be minimized!

Status: Freeware with full source.

Installation: Choose 'Install Component...' at the main menu.
              Choose the 'Into new package' tab. Select in
              the 'Unit file name' the minmodal.pas. And in the
              'Package file name' input any file name
              (not 'minmodal' - 'minmodalpack', for example).
              Press 'Ok' button. That's all!

---------- Other products ----------

MaxSpace
~~~~~~~~
MaxSpace just makes Delphi and C++ Builder more comfortable in usage.
It turns the IDE ToolBar and ObjectInspector into emerging state,
giving you unbelievable freedom of program editing and form designing.
For more information visit http://www.zecos.com/maxspc

ElegantMDI
~~~~~~~~~~
This component is a new elegant realiztion of the old MDI interface.
All windows (MDIChild) will be presented as buttons on autohidden panel.
User can toggle between windows without choosing items in main menu.
It looks very effectively. Freeware.

WavePanel
~~~~~~~~~
WavePanel - component with running colors, like panel at the bottom
of the startup screen when Windows 9x loading. Freeware.

All this programs you can download from http://www.zecos.com

Zecos Software team
http://www.zecos.com
support@zecos.com
}

{$WARN SYMBOL_DEPRECATED OFF}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TMinModal = class(TComponent)
  private
    Form: TForm;
    OldModalWndProc: Pointer;
    InstanceModalWndProc: Pointer;
    FAbout: String;
    FOnMinimize: TNotifyEvent;
    procedure SetAbout(const Value: String);
  protected
    procedure ModalWndProc(var Message: TMessage); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property About: String read FAbout write SetAbout stored False;
    property OnMinimize: TNotifyEvent read FOnMinimize write FOnMinimize;
  end;

procedure Register;

implementation

uses AboutMinModal;

const
//  EMail = 'support@zecos.com';
  WebHome = 'http://www.zecos.com';

procedure Register;
begin
  RegisterComponents('Samples', [TMinModal]);
end;

procedure RememberMe(Sender: TObject);
begin
  with TFormAboutMinModal.Create(Application) do begin
    WavePanel5.Caption := WebHome;
    Caption := WebHome;
    Left := (Screen.Width-Width) div 2;
    Top  := (Screen.Height-Height) div 2;
    ShowModal;
    Free;
  end;
end;

{ TMinModal }

constructor TMinModal.Create(AOwner: TComponent);
var
  Component: TComponent;
begin
  inherited Create(AOwner);
  FAbout := 'Minimizer For Modal Windows ('+WebHome+')';

  if not(csDesigning in ComponentState) then begin
    Component:=Owner;
    while Assigned(Component) and not (Component is TForm) do
      Component:= Component.Owner;
    if not Assigned(Component) then
      Exit;
    Form := Component as TForm;

    if (not (csDesigning in ComponentState)) then begin
      InstanceModalWndProc := MakeObjectInstance(ModalWndProc);
      OldModalWndProc := Pointer(GetWindowLong(Form.Handle, GWL_WNDPROC));
      SetWindowLong(Form.Handle, GWL_WNDPROC, Longint(InstanceModalWndProc));
    end;
  end;
end;

destructor TMinModal.Destroy;
begin
  if Assigned(InstanceModalWndProc) then begin
    SetWindowLong(Form.Handle, GWL_WNDPROC, Longint(OldModalWndProc));
    FreeObjectInstance(InstanceModalWndProc);
    InstanceModalWndProc := nil;
  end;
  inherited Destroy;
end;

procedure TMinModal.ModalWndProc(var Message: TMessage);
begin
  case Message.Msg of
    WM_SYSCOMMAND:
      begin
        if Message.WParam=SC_MINIMIZE then begin
          if Assigned(FOnMinimize) then
            FOnMinimize(Self);
          ShowWindow(Application.Handle, SW_SHOWMINNOACTIVE);
          Exit;
        end;
      end;
    WM_DESTROY:
      begin
        if Assigned(InstanceModalWndProc) then begin
          SetWindowLong(Form.Handle, GWL_WNDPROC, Longint(OldModalWndProc));
          FreeObjectInstance(InstanceModalWndProc);
          InstanceModalWndProc := nil;
        end;
      end;
    WM_ACTIVATE:
      begin
        if Message.WParamLo<>WA_INACTIVE then begin
          ShowWindow(Application.Handle, SW_SHOWNORMAL);
//          Exit;
        end;
      end;
  end;

  Message.Result:= CallWindowProc(OldModalWndProc, Form.Handle, Message.Msg, Message.WParam, Message.LParam);
end;

procedure TMinModal.SetAbout(const Value: String);
begin
  if FAbout<>Value then RememberMe(Self);
end;

end.
